#!/usr/bin/env python
# -*- coding: utf8 -*-

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import sys
import os
import glob
from sos.plugins import Plugin, RedHatPlugin

class ovm3(Plugin,RedHatPlugin):
    """ Gather system information for OVS systems """
    
    plugin_name = 'ovm3'

    def setup(self):
        self.add_copy_spec([
            "/var/log/osmd/*",
            "/var/log/*",
            "/etc/ntp.conf",
            "/root/anaconda-ks.cfg",
            "/etc/sysconfig/osmd",
            "/var/log/ovs-agent.log",
        ])
	 
        for vm in glob.glob('/OVS/Repositories/*/.ovsrepo'):
                self.add_cmd_output("/bin/cat "+vm, suggest_filename=vm)
        for filel in glob.glob("/var/log/messages*"):
                self.add_copy_spec(filel)
        for filel in glob.glob("/var/run/ovs-agent/*.lock*"):
                self.add_cmd_output(filel)

        self.add_cmd_output("/bin/cat /root/.bash_history",suggest_filename="bash_history")
        self.add_cmd_output("/bin/mount | egrep -e 'automount|pid.*nfs' 2>&1", suggest_filename="auto_nfs_check")
        self.add_cmd_output("service ntpd status", suggest_filename="ntpd.status")
        self.add_cmd_output("/usr/sbin/xm info", suggest_filename="xm.info")
        self.add_cmd_output("/usr/sbin/xm list", suggest_filename="xm.list")
        self.add_cmd_output("/usr/sbin/xm list -l", suggest_filename="xm.list-l")
        self.add_cmd_output("/usr/sbin/xm dmesg", suggest_filename="xm.dmesg")
        self.add_cmd_output("/usr/sbin/xm vcpu-list", suggest_filename="xm.vcpu-list")
        self.add_cmd_output("/usr/sbin/xm sched-credit", suggest_filename="xm.sched-credit")
        self.add_cmd_output("/usr/sbin/xenpm get-cpuidle-states 2>&1", suggest_filename="xenpm.get-cpuidle-states")
        self.add_cmd_output("/usr/sbin/xenpm get-cpufreq-states", suggest_filename="xenpm.get-cpufreq-states")
        self.add_cmd_output("/usr/sbin/xenpm get-cpuidle-states | grep total | uniq 2>&1", suggest_filename="get_cpuidle_states_uniq")
        self.add_cmd_output("/usr/sbin/xenpm get-cpufreq-para", suggest_filename="xenpm.get-cpufreq-para")
        self.add_cmd_output("xenstore-ls", suggest_filename="xenstore-ls")
        self.add_cmd_output("service xend status", suggest_filename="xend.status")
        self.add_cmd_output("dominfo -c -f", suggest_filename="dominfo-c-f")
        self.add_cmd_output("dom0memcontroller --dumpMemInfo", suggest_filename="dom0memcontroller--dumpMemInfo")
        self.add_cmd_output('find /OVS -type f -name "*.cfg" | while read -r line; do echo "====================================";cat "$line"; echo "===================================="; done', suggest_filename="vms.configs")
        self.add_cmd_output("rpm -qa | grep -E \(xen\|ovs\|ocfs\) | sort", suggest_filename="rpm_qa_xen_ovs_ocfs")
        self.add_cmd_output("/bin/ls -l /OVS/Repositories/*/VirtualDisks", suggest_filename="listvdisks")
        self.add_cmd_output("/bin/ls -l -R /poolfsmnt", suggest_filename="ls-l-R_poolfsmnt")
        self.add_cmd_output("/bin/ls -l -R /nfsmnt", suggest_filename="ls-l-R_nfsmnt")
        self.add_cmd_output("/bin/ls -l -R /dlm", suggest_filename="ls-l-R-dlm")
        self.add_cmd_output("/usr/sbin/brctl show", suggest_filename="brctl.show")
        self.add_cmd_output("ip -s -s -s link show", suggest_filename="ip-s-s-s-link-show")
        self.add_cmd_output("ip address show", suggest_filename="ip-address-show")
        self.add_cmd_output("ps -elfy", suggest_filename="ps-elfy")
        self.add_cmd_output("rpm -qa --queryformat '%{installtime} %{name}-%{version}-%{release} %{installtime:date}\n'", suggest_filename="rpm_qa_installtime")
        self.add_cmd_output("lsof")
        self.add_cmd_output("/sbin/ip route show", suggest_filename="ip_route_show")
        self.add_cmd_output("/sbin/fdisk -l", suggest_filename="fdisk_l")
        self.add_cmd_output("/sbin/arp -na", suggest_filename="arp_na")
        self.add_cmd_output("/sbin/ip -s neighbor list", suggest_filename="ip_-s_neighbor_list")
        self.add_cmd_output("/bin/cat /proc/net/arp", suggest_filename="cat_proc_net_arp")
        self.add_cmd_output("/usr/sbin/ovs-agent-db dump_db server 2>&1", suggest_filename="dump_db_server")
        self.add_cmd_output("/usr/sbin/ovs-agent-db dump_db aproc 2>&1", suggest_filename="dump_db_aproc")
        self.add_cmd_output("/usr/sbin/ovs-agent-db dump_db repository 2>&1", suggest_filename="dump_db_repository")
        self.add_cmd_output("/usr/sbin/ovs-agent-db -c dump_db server_pool 2>&1",suggest_filename="dump_db_serverpool")
        self.add_cmd_output("/usr/sbin/ovs-agent-db -c dump_db server_pool_servers  2>&1",suggest_filename="dump_db_server_pool_servers")
        self.add_cmd_output("/usr/sbin/ovs-agent-db -c dump_db monitored_vms  2>&1",suggest_filename="dump_db_monitored_vms")
        self.add_cmd_output("ntpq -pn", suggest_filename="ntpq_-pn")
        self.add_cmd_output("pgrep ntpd", suggest_filename="pgrep_ntpd")
        self.add_cmd_output("netstat -tulpn | grep 123", suggest_filename="netstat_-tulpn_grep_123")
        self.add_cmd_output("grep -i ntp /var/log/messages", suggest_filename="ntp_messages")
        
        for bond in glob.glob('/proc/net/bonding/bond*'):
                self.add_cmd_output("/bin/cat "+bond, suggest_filename="proc_net_"+bond)
        self.add_cmd_output("multipath -ll", suggest_filename="multipath-ll")
        self.add_cmd_output('find /sys/class/net/* -printf "%f\n"  | while read -r line; do echo "$line ======================="; ethtool -i "$line"; echo "====================================";done',suggest_filename="ethtool")
        self.add_cmd_output("/bin/cat /u01/app/oracle/ovm-manager-3/.config", suggest_filename="ovm-manager_3_config")
        self.add_cmd_output("/bin/cat /etc/ovs-config", suggest_filename="ovsconfig")
        self.add_cmd_output("/bin/cat /etc/ovs-info", suggest_filename="ovsinfo")
        self.add_cmd_output("/bin/cat /etc/ovs-release", suggest_filename="ovsrelease")

	if os.path.exists("/etc/sysconfig/oswatcher.cfg"):
            oswconf = "/etc/sysconfig/oswatcher.cfg"
            oswfile = open(oswconf, 'r')
            for line in ovwfile.readlines():
                if line.startswith("DATADIR"):
                    label,dir = line.split("=")
            oswfile.close()
            if dir:
                self.add_copy_spec(dir)
        
        if os.path.exists("/usr/sbin/vmpstat.sh"):
            self.add_cmd_output("/usr/sbin/vmpstat.sh -S", suggest_filename="vmpstat.out")
        
        if os.path.exists("/usr/sbin/ocfs2hc"):
            self.add_cmd_output("/usr/sbin/ocfs2hc --json-output", suggest_filename="ocfs2hc.json")
            self.add_cmd_output("/usr/sbin/ocfs2hc --validate-cluster --json-output", suggest_filename="ocfs2hc_validate.json")
            self.add_cmd_output("/usr/sbin/ocfs2hc --cluster-report --json-output", suggest_filename="ocfs2hc_report.json")

        if os.path.exists("/var/log/xsigo"):
            for xsigof in glob.glob("/var/log/xsigo/xsadmdsh.lo*"):
                self.add_copy_spec(xsigof)
            for ovmw in glob.glob("ovmwatch.lo*"):
                self.add_copy_spec(ovmw)
            for osc in glob.glob("osc.lo*"):
                self.add_copy_spec(osc)
            for ovmc in glob.glob("/var/log/ovm-consoled.lo*"):
                self.add_copy_spec(ovmc)
            for brcm in glob.glob("/var/log/brcm-iscsi.lo*"):
                self.add_copy_spec(brcm)
            self.add_cmd_output("lsmod  | grep xsv", suggest_filename="xsigo_kernel_modules")
